/*
    The interface_to_ip function has been copied from the MediaTomb 
    Project and adapted to plain C. Original Copyright below:
       
    MediaTomb - http://www.mediatomb.cc/
    
    tools.cc - this file is part of MediaTomb.
    
    Copyright (C) 2005 Gena Batyan <bgeradz@mediatomb.cc>,
                       Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
    
    Copyright (C) 2006-2010 Gena Batyan <bgeradz@mediatomb.cc>,
                            Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>,
                            Leonhard Wimmer <leo@mediatomb.cc>
    
    MediaTomb is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2
    as published by the Free Software Foundation.
    
    MediaTomb is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    version 2 along with MediaTomb; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <string.h>
#include <stdio.h>

char *interface_to_ip(const char *interface)
{
    struct if_nameindex *iflist = NULL;
    struct if_nameindex *iflist_free = NULL;
    struct ifreq if_request;
    struct sockaddr_in local_address;
    int local_socket;

    if ((interface == NULL) || (strlen(interface) == 0)) 
    {
        return NULL;
    }

    local_socket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if (local_socket < 0)
    {
        fprintf(stderr, "Could not create local socket: %s\n", strerror(errno));
        return NULL;
    }

    iflist = iflist_free = if_nameindex();
    if (iflist == NULL)
    {
        fprintf(stderr, "Could not get interface list: %s\n", strerror(errno));
        close(local_socket);
        return NULL;
    }

    while (iflist->if_index || iflist->if_name)
    {
        if (strcmp(interface, iflist->if_name) == 0)
        {
            strncpy(if_request.ifr_name, iflist->if_name, IF_NAMESIZE);
            if (ioctl(local_socket, SIOCGIFADDR, &if_request) != 0)
            {
                fprintf(stderr, "Could not determine interface address: %s\n",
                        strerror(errno));
                close(local_socket);
                if_freenameindex(iflist_free);
                return NULL;
            }

            memcpy(&local_address, &if_request.ifr_addr, sizeof(if_request.ifr_addr));
            char *ip = inet_ntoa(local_address.sin_addr);
            if_freenameindex(iflist_free);
            close(local_socket);
            if (ip != NULL)
            {
                return (char *)strdup(ip);
            } 
            else
            {
                return NULL;
            }
        }
        iflist++;
    }

    close(local_socket);
    if_freenameindex(iflist_free);
    return NULL;
}

